%Demonstration of ICA using artificial signals. 
%
%2002-12-07 | Michael Vinther | mv@logicnet.dk
clear all,close all

% Original signals, columns are components, rows are observations (=samples)
T = 1000;
S = [sin(linspace(0,100,T))' mod(linspace(0,20,T),3)' cos(linspace(0,10,T))' sign(sin(linspace(0,60,T)))'+0.5];
for i=1:size(S,2)
  S(:,i) = 1/std(S(:,i))*S(:,i); % Normalize
end
figure
plotcomponents(S,'s');
title('S: Original source');

% Transformation matrix
A = [1 2 3 1; 3 1 2 2; 1 3 1 3;1  -1 -2 1]

% Transformed signals
X = (A*S')';
figure
plotcomponents(X,'x');
title('X: Mixed signals');

% Reconstruct using ICA result
[Y,W,P] = ica(X,'kurt',[100,10e-6,1]); % Use optimized parameters
%[Y,W,P] = ica(X); % Use default parameters

% Show convergence
figure
plot(P)
axis tight
xlabel('Iteration')
title('Convergence')

% Show solution
figure
plotcomponents(Y,'y');
title('Y: S reconstructed from X');
EstimatedA = inv(W)
disp('Note that columns can be swapped and their sign inverted')
